import ast
import os
from pcas import keyvalue
import numpy as np
from sklearn.preprocessing import StandardScaler
from sklearn.model_selection import train_test_split,  GridSearchCV
from sklearn import svm
from sklearn.decomposition import PCA
from sklearn.metrics import accuracy_score, confusion_matrix
from sklearn.pipeline import Pipeline
import matplotlib.pyplot as plt
import seaborn as sns

# Connect to the table
conn = keyvalue.Connection('ml_polydim', address='fanosearch.net:12356')
t = conn.connect_to_table('balanced_quasiperiod')
# Prepare to read the data
template = {'ehrhart': '', 'quasiperiod':0}
target_dim = os.environ["TARGET_DIM"]
this_dim = []
# Read in the data and convert 'ehrhart' to a NumPy array
for d in t.select(template, f'WHERE dimension = {target_dim}'):
    this_dim.append({
        'ehrhart': np.array(ast.literal_eval(d['ehrhart'])),
        'quasiperiod': d['quasiperiod'],
    })
# Clean up
t.close()
conn.close()
# Move to PCA co-ordinates
X = np.array([x['ehrhart'] for x in this_dim])
y = np.array([x['quasiperiod'] for x in this_dim])
pca = PCA()
pca.fit(X)
X_new = pca.fit_transform(X)
# Split the data into training set and test set
X_train, X_test, y_train, y_test = train_test_split(X_new, y, test_size=0.5, shuffle=True,  stratify=y)

# Make pipeline with SVM and standard scaler
pipe_SVM = Pipeline([
    ('scaler', StandardScaler()),
    ('SVM', svm.LinearSVC(random_state = 42))
    ])

# Parameter grid for SVM
grid_params_SVM = [{'SVM__C': [0.001,0.005,0.01,0.05,0.1,1,5,10]}]

# Cross validation to find the best parameter
CV_SVM = GridSearchCV(
    estimator = pipe_SVM,
    param_grid = grid_params_SVM,
    cv = 5,
    return_train_score=True, 
    verbose=0,
    n_jobs=4)

# Train
CV_SVM.fit(X_train, y_train)

# Predict (using the best parameters)
predictions=CV_SVM.predict(X_test)

# Compute confusion matrices
mat_true=confusion_matrix(predictions, y_test, normalize='true')
mat_pred=confusion_matrix(predictions, y_test, normalize='pred')
mat_raw=confusion_matrix(predictions, y_test)

# Print and save the results
with open(f'quasiperiod_svm_hyperparameter_tuning_dim_{target_dim}', 'w') as f:
    print(f'The best parameters are: {CV_SVM.best_params_}')
    print(f'The best parameters are: {CV_SVM.best_params_}', file=f)
    a=accuracy_score(y_test, predictions)
    print(f'Accuracy: {a}')
    print(f'Accuracy: {a}', file=f)
    print('True confusion matrix:', file=f)
    print(np.array2string(mat_true, separator=','), file=f)
    print('Predicted confusion matrix:', file=f)
    print(np.array2string(mat_pred, separator=','), file=f)
    print('Raw confusion matrix:', file=f)
    print(np.array2string(mat_raw, separator=','), file=f)
